<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * Class AdminKiwizController
 */
class KiwizInvoiceHandlerFrontModuleFrontController extends ModuleFrontController
{
    /**
     * @var
     */
    public $errors;
    /**
     * @var
     */
    public $action;

    public function __construct()
    {
        parent::__construct();

        $this->tpl_path_local = $this->module->getLocalPath();
        if (file_exists(_PS_THEME_DIR_ . 'modules/kiwizinvoicehandler/views/templates/')) {
            $this->tpl_path = _PS_THEME_DIR_ . 'modules/kiwizinvoicehandler/';
        } else {
            $this->tpl_path = $this->tpl_path_local;
        }
    }

    /**
     *
     */
    public function postProcess()
    {
        parent::postProcess();
        if (!$this->initProcess()) {
            $this->initContent();
        }
//        exit;
    }

    /**
     *
     */
    public function initProcess()
    {
        $this->checkCacheFolder();

        if ($action = Tools::getValue('submitAction')) {
            if (!$this->$action()) {
                return false;
            }
        } else {
            $this->errors[] = Tools::displayError('You do not have permission to view this.');

            return false;
        }

        return true;
    }

    /**
     *
     */
    public function checkCacheFolder()
    {
        if (!is_dir(_PS_CACHE_DIR_ . 'tcpdf/')) {
            mkdir(_PS_CACHE_DIR_ . 'tcpdf/');
        }
    }

    public function initContent()
    {
        parent::initContent();
        if ($this->errors) {
            $this->context->smarty->assign('errors', $this->errors);
            $this->setTemplate('errors.tpl');
        }
    }

    /**
     * Get invoice form api
     */
    public function generateInvoicePdf()
    {
        if (!$this->checkPermissions()) {
            return false;
        }

        $blockHash = Tools::getValue('block_hash');
        $documentId = Tools::getValue('document_id');

        $document = new OrderInvoice((int)$documentId);
        $fileName = $document->getInvoiceNumberFormatted($this->context->language->id);

        $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi();
        try {
            $pdf = $kiwizApi->get(
                'invoice',
                array(
                    'name' => 'block_hash',
                    'value' => $blockHash
                )
            );
            if (is_object($pdf)) {
                $apiError = new ApiError();
                $apiError->setIdOrderFile($documentId);
                $apiError->setFileType('invoice');
                $apiError->setCodeError($pdf->getCode());
                $apiError->setMessage($pdf->getMessage());
                $apiError->save();
                $this->errors[] = Tools::displayError($pdf->getCode() . ' ' . $pdf->getMessage());
            } else {
                $pdfMerger = new PDFMerger();
                file_put_contents(_PS_ROOT_DIR_ . '/upload/' . $fileName . '.pdf', $pdf);
                $path = _PS_ROOT_DIR_ . '/upload/' . $fileName . '.pdf';
                try {
                    $pdfMerger->addPDF($path);
                } catch (Exception $e) {
                    $this->errors[] = Tools::displayError($e->getMessage());

                    return false;
                }
                $pdfMerger->merge('browser', $fileName, 'P');
            }
        } catch (Exception $e) {
            $this->errors[] =
                Tools::displayError('The order block_hash -- or the invoice order block_hash -- is missing.');

            return false;
        }
    }

    public function checkPermissions()
    {
        $cookie = $this->context->cookie;
        if (!$cookie->isLogged()) {
            $this->errors[] = Tools::displayError('You must be logged to access this page');

            return false;
        }

        if (!Tools::isSubmit('block_hash')
            || !Tools::isSubmit('document_id')
            || !Tools::isSubmit('customer_id')) {
            $this->errors[] = Tools::displayError('Information missing');

            return false;
        }

        $currentCustomerId = $this->context->customer->id;
        $customerId = Tools::getValue('customer_id');
        if ($currentCustomerId != $customerId) {
            $this->errors[] = Tools::displayError('Information missing');
            return false;
        }

        return true;
    }

    /**
     * Get slip from api
     */

    public function generateOrderSlipPdf()
    {
        if (!$this->checkPermissions()) {
            return false;
        }

        $blockHash = Tools::getValue('block_hash');
        $documentId = Tools::getValue('document_id');

        $fileName = '0000' . $documentId . '.pdf';

        $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi();
        try {
            $pdf = $kiwizApi->get(
                'creditmemo',
                array(
                    'name' => 'block_hash',
                    'value' => $blockHash
                )
            );
            if (is_object($pdf)) {
                $apiError = new ApiError();
                $apiError->setIdOrderFile($documentId);
                $apiError->setFileType('slip');
                $apiError->setCodeError($pdf->getCode());
                $apiError->setMessage($pdf->getMessage());
                $apiError->save();
                $this->errors[] = Tools::displayError($pdf->getCode() . ' ' . $pdf->getMessage());
            } else {
                $pdfMerger = new PDFMerger();
                file_put_contents(_PS_ROOT_DIR_ . '/upload/' . $fileName . '.pdf', $pdf);
                $path = _PS_ROOT_DIR_ . '/upload/' . $fileName . '.pdf';
                try {
                    $pdfMerger->addPDF($path);
                } catch (Exception $e) {
                    $this->errors[] = Tools::displayError($e->getMessage());

                    return false;
                }
                $pdfMerger->merge('browser', $fileName, 'P');
            }
        } catch (Exception $e) {
            $this->errors[] =
                Tools::displayError('The order block_hash -- or the slip order block_hash -- is missing.');

            return false;
        }
    }

    public function cron()
    {
        $this->cronInvoices();
        $this->cronSlips();
        die;
    }

    /**
     * Update Invoices from api
     */
    public function cronInvoices()
    {
        $tableOrder = _DB_PREFIX_ . 'orders';
        $tableInvoice = _DB_PREFIX_ . "order_invoice";
        $tableKiwizData = _DB_PREFIX_ . "kiwiz_file_data";
        $sql = "SELECT * FROM  $tableInvoice";
        $sql .= " LEFT JOIN $tableKiwizData ON $tableInvoice.id_order_invoice = $tableKiwizData.id_order_file";
        $sql .= " LEFT JOIN $tableOrder ON $tableInvoice.id_order = $tableOrder.id_order";
        $sql .= " WHERE $tableKiwizData.file_type = 'invoice'";
        $sql .= " AND $tableKiwizData.status = '0'";
        $sql .= " AND $tableOrder.id_shop = " . (int)Context::getContext()->shop->id;

        $results = Db::getInstance()->executeS($sql);

        foreach ($results as $result) {
            $invoice = new OrderInvoice($result['id_order_invoice']);
            $invoice->update();
        }
        $this->vd('cron done');
    }

    /**
     * @param $var
     */
    public function vd($var)
    {
        echo '<pre style="color:#fff;background: #211f1f;padding: 10px;">';
        var_dump($var);
        echo '</pre>';
    }

    /**
     * Update Slips from api
     */
    public function cronSlips()
    {
        $tableOrder = _DB_PREFIX_ . 'orders';
        $tableSlip = _DB_PREFIX_ . "order_slip";
        $tableKiwizData = _DB_PREFIX_ . "kiwiz_file_data";
        $sql = "SELECT * FROM  $tableSlip";
        $sql .= " LEFT JOIN $tableKiwizData ON $tableSlip.id_order_slip = $tableKiwizData.id_order_file";
        $sql .= " LEFT JOIN $tableOrder ON $tableSlip.id_order = $tableOrder.id_order";
        $sql .= " WHERE $tableKiwizData.file_type = 'slip'";
        $sql .= " AND $tableKiwizData.status = '0'";
        $sql .= " AND $tableOrder.id_shop = " . (int)Context::getContext()->shop->id;

        $results = Db::getInstance()->executeS($sql);

        foreach ($results as $result) {
            $slip = new OrderSlip($result['id_order_slip']);
            $slip->update();
        }
        $this->vd('slips done');
    }
}
