<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author PrestaShop SA <contact@prestashop.com>
 * @copyright  2007-2018 PrestaShop SA
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class OrderHistory extends OrderHistoryCore
{
    public function sendEmail($order, $template_vars = false)
    {
        $result = Db::getInstance()->getRow('
			SELECT osl.`template`,
			 c.`lastname`,
			  c.`firstname`,
			   osl.`name` AS osname,
			    c.`email`,
			     os.`module_name`,
			      os.`id_order_state`,
			       os.`pdf_invoice`,
			        os.`pdf_delivery`
			FROM `' . _DB_PREFIX_ . 'order_history` oh
				LEFT JOIN `' . _DB_PREFIX_ . 'orders` o ON oh.`id_order` = o.`id_order`
				LEFT JOIN `' . _DB_PREFIX_ . 'customer` c ON o.`id_customer` = c.`id_customer`
				LEFT JOIN `' . _DB_PREFIX_ . 'order_state` os ON oh.`id_order_state` = os.`id_order_state`
				LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` osl ON (os.`id_order_state` = osl.`id_order_state` 
				AND osl.`id_lang` = o.`id_lang`)
			WHERE oh.`id_order_history` = ' . (int)$this->id . ' AND os.`send_email` = 1');
        if (isset($result['template']) && Validate::isEmail($result['email'])) {
            ShopUrl::cacheMainDomainForShop($order->id_shop);
            $topic = $result['osname'];
            $data = array(
                '{lastname}' => $result['lastname'],
                '{firstname}' => $result['firstname'],
                '{id_order}' => (int)$this->id_order,
                '{order_name}' => $order->getUniqReference()
            );
            if ($result['module_name']) {
                $module = Module::getInstanceByName($result['module_name']);
                if (Validate::isLoadedObject($module)
                    && isset($module->extra_mail_vars) && is_array($module->extra_mail_vars)) {
                    $data = array_merge($data, $module->extra_mail_vars);
                }
            }
            if ($template_vars) {
                $data = array_merge($data, $template_vars);
            }

            $data['{total_paid}'] =
                Tools::displayPrice((float)$order->total_paid, new Currency((int)$order->id_currency), false);

            if (Validate::isLoadedObject($order)) {
                if (($result['pdf_invoice'] || $result['pdf_delivery'])) {
                    $context = Context::getContext();
                    $invoice = $order->getInvoicesCollection();
                    $file_attachment = array();

                    if ($result['pdf_invoice'] && (int)Configuration::get('PS_INVOICE') && $order->invoice_number) {
                        Hook::exec('actionPDFInvoiceRender', array('order_invoice_list' => $invoice));
                        $pdf = new PDF($invoice, PDF::TEMPLATE_INVOICE, $context->smarty);

                        if (Configuration::get('KIWIZINVOICEHANDLER_LIVE_MODE')) {
                            $document = $order->getMainInvoice();

                            if ($document->getKiwizData()) {
                                $blockHash = $document->getKiwizData()->getBlockHash();
                                $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi();
                                $resultApi = $kiwizApi->get(
                                    'invoice',
                                    array(
                                        'name' => 'block_hash',
                                        'value' => $blockHash
                                    )
                                );
                                if (is_object($resultApi)) {
                                    $content = $pdf->render(false);
                                } else {
                                    $content = $resultApi;
                                }
                            } else {
                                $content = $pdf->render(false);
                            }
                        } else {
                            $content = $pdf->render(false);
                        }
                        $file_attachment['invoice']['content'] = $content;
                        $file_attachment['invoice']['name'] =
                            Configuration::get(
                                'PS_INVOICE_PREFIX',
                                (int)$order->id_lang,
                                null,
                                $order->id_shop
                            ) . sprintf('%06d', $order->invoice_number) . '.pdf';
                        $file_attachment['invoice']['mime'] = 'application/pdf';
                    }

                    if ($result['pdf_delivery'] && $order->delivery_number) {
                        $pdf = new PDF($invoice, PDF::TEMPLATE_DELIVERY_SLIP, $context->smarty);
                        $file_attachment['delivery']['content'] = $pdf->render(false);
                        $file_attachment['delivery']['name'] =
                            Configuration::get(
                                'PS_DELIVERY_PREFIX',
                                Context::getContext()->language->id,
                                null,
                                $order->id_shop
                            ) . sprintf('%06d', $order->delivery_number) . '.pdf';
                        $file_attachment['delivery']['mime'] = 'application/pdf';
                    }
                } else {
                    $file_attachment = null;
                }
                if (!Mail::Send(
                    (int)$order->id_lang,
                    $result['template'],
                    $topic,
                    $data,
                    $result['email'],
                    $result['firstname'] . ' ' . $result['lastname'],
                    null,
                    null,
                    $file_attachment,
                    null,
                    _PS_MAIL_DIR_,
                    false,
                    (int)$order->id_shop
                )) {
                    return false;
                }
            }
            ShopUrl::resetMainDomainCache();
        }
        return true;
    }
}
