<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_ . 'kiwizinvoicehandler/classes/FileData.php';

class OrderSlip extends OrderSlipCore
{
    public $id;

    private $file_type = 'slip';

    public function __construct($id = null, $id_lang = null, $id_shop = null)
    {
        parent::__construct($id, $id_lang, $id_shop);
    }

    public static function create(
        Order $order,
        $product_list,
        $shipping_cost = false,
        $amount = 0,
        $amount_choosen = false,
        $add_tax = true
    ) {
        $currency = new Currency((int)$order->id_currency);
        $order_slip = new OrderSlip();
        $order_slip->id_customer = (int)$order->id_customer;
        $order_slip->id_order = (int)$order->id;
        $order_slip->conversion_rate = $currency->conversion_rate;

        if ($add_tax) {
            $add_or_remove = 'add';
            $inc_or_ex_1 = 'excl';
            $inc_or_ex_2 = 'incl';
        } else {
            $add_or_remove = 'remove';
            $inc_or_ex_1 = 'incl';
            $inc_or_ex_2 = 'excl';
        }

        $order_slip->{'total_shipping_tax_' . $inc_or_ex_1} = 0;
        $order_slip->{'total_shipping_tax_' . $inc_or_ex_2} = 0;
        $order_slip->partial = 0;

        if ($shipping_cost !== false) {
            $order_slip->shipping_cost = true;
            $carrier = new Carrier((int)$order->id_carrier);
            $address = Address::initialize($order->id_address_delivery, false);
            $tax_calculator = $carrier->getTaxCalculator($address);
            $order_slip->{'total_shipping_tax_' . $inc_or_ex_1} =
                ($shipping_cost === null ? $order->{'total_shipping_tax_' . $inc_or_ex_1} : (float)$shipping_cost);

            if ($tax_calculator instanceof TaxCalculator) {
                $order_slip->{'total_shipping_tax_' . $inc_or_ex_2} =
                    Tools::ps_round(
                        $tax_calculator->{$add_or_remove . 'Taxes'}(
                            $order_slip->{'total_shipping_tax_' . $inc_or_ex_1}
                        ),
                        _PS_PRICE_COMPUTE_PRECISION_
                    );
            } else {
                $order_slip->{'total_shipping_tax_' . $inc_or_ex_2} =
                    $order_slip->{'total_shipping_tax_' . $inc_or_ex_1};
            }
        } else {
            $order_slip->shipping_cost = false;
        }

        $order_slip->amount = 0;
        $order_slip->{'total_products_tax_' . $inc_or_ex_1} = 0;
        $order_slip->{'total_products_tax_' . $inc_or_ex_2} = 0;

        foreach ($product_list as &$product) {
            $order_detail = new OrderDetail((int)$product['id_order_detail']);
            $price = (float)$product['unit_price'];
            $price = (float)number_format($price, 2);
            $quantity = (int)$product['quantity'];
            $order_slip_resume = OrderSlip::getProductSlipResume((int)$order_detail->id);

            if ($quantity + $order_slip_resume['product_quantity'] > $order_detail->product_quantity) {
                $quantity = $order_detail->product_quantity - $order_slip_resume['product_quantity'];
            }

            if ($quantity == 0) {
                continue;
            }

            if (!Tools::isSubmit('cancelProduct') && $order->hasBeenPaid()) {
                $order_detail->product_quantity_refunded += $quantity;
            }

            $order_detail->save();

            $address = Address::initialize($order->id_address_invoice, false);
            $id_address = (int)$address->id;
            $id_tax_rules_group = Product::getIdTaxRulesGroupByIdProduct((int)$order_detail->product_id);
            $tax_calculator = TaxManagerFactory::getManager($address, $id_tax_rules_group)->getTaxCalculator();

            $order_slip->{'total_products_tax_' . $inc_or_ex_1} += $price * $quantity;
            $total_products = array();

            if (in_array(Configuration::get('PS_ROUND_TYPE'), array(Order::ROUND_ITEM, Order::ROUND_LINE))) {
                if (!isset($total_products[$id_tax_rules_group])) {
                    $total_products[$id_tax_rules_group] = 0;
                }
            } else {
                if (!isset($total_products[$id_tax_rules_group . '_' . $id_address])) {
                    $total_products[$id_tax_rules_group . '_' . $id_address] = 0;
                }
            }

            $product_tax_incl_line = Tools::ps_round(
                $tax_calculator->{$add_or_remove . 'Taxes'}($price) * $quantity,
                _PS_PRICE_COMPUTE_PRECISION_
            );

            switch (Configuration::get('PS_ROUND_TYPE')) {
                case Order::ROUND_ITEM:
                    $product_tax_incl =
                        Tools::ps_round(
                            $tax_calculator->{$add_or_remove . 'Taxes'}($price),
                            _PS_PRICE_COMPUTE_PRECISION_
                        ) * $quantity;
                    $total_products[$id_tax_rules_group] += $product_tax_incl;
                    break;
                case Order::ROUND_LINE:
                    $product_tax_incl = $product_tax_incl_line;
                    $total_products[$id_tax_rules_group] += $product_tax_incl;
                    break;
                case Order::ROUND_TOTAL:
                    $product_tax_incl = $product_tax_incl_line;
                    $total_products[$id_tax_rules_group . '_' . $id_address] += $price * $quantity;
                    break;
            }

            $product['unit_price_tax_' . $inc_or_ex_1] = $price;
            $product['unit_price_tax_' . $inc_or_ex_2] = Tools::ps_round(
                $tax_calculator->{$add_or_remove . 'Taxes'}($price),
                _PS_PRICE_COMPUTE_PRECISION_
            );
            $product['total_price_tax_' . $inc_or_ex_1] =
                Tools::ps_round($price * $quantity, _PS_PRICE_COMPUTE_PRECISION_);
            $product['total_price_tax_' . $inc_or_ex_2] =
                Tools::ps_round($product_tax_incl, _PS_PRICE_COMPUTE_PRECISION_);
        }

        unset($product);

        foreach ($total_products as $key => $price) {
            if (Configuration::get('PS_ROUND_TYPE') == Order::ROUND_TOTAL) {
                $tmp = explode('_', $key);
                $address = Address::initialize((int)$tmp[1], true);
                $tax_calculator = TaxManagerFactory::getManager($address, $tmp[0])->getTaxCalculator();
                $order_slip->{'total_products_tax_' . $inc_or_ex_2} +=
                    Tools::ps_round(
                        $tax_calculator->{$add_or_remove . 'Taxes'}($price),
                        _PS_PRICE_COMPUTE_PRECISION_
                    );
            } else {
                $order_slip->{'total_products_tax_' . $inc_or_ex_2} += $price;
            }
        }

        $order_slip->{'total_products_tax_' . $inc_or_ex_2} -= (float)$amount && !$amount_choosen ? (float)$amount : 0;
        $order_slip->amount = $amount_choosen ? (float)$amount : $order_slip->{'total_products_tax_' . $inc_or_ex_1};
        $order_slip->shipping_cost_amount = $order_slip->total_shipping_tax_incl;

        if ((float)$amount && !$amount_choosen) {
            $order_slip->order_slip_type = 1;
        }
        if (((float)$amount && $amount_choosen) || $order_slip->shipping_cost_amount > 0) {
            $order_slip->order_slip_type = 2;
        }

        if (!$order_slip->add()) {
            return false;
        }

        $res = true;

        foreach ($product_list as $product) {
            $res &= $order_slip->addProductOrderSlip($product);
        }
        if (Configuration::get('KIWIZINVOICEHANDLER_LIVE_MODE')) {
            $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi(Context::getContext()->smarty);
            $apiResponse = $kiwizApi->save('creditmemo', $order_slip);

            $fileData = new FileData();
            $fileData->setIdOrderFile($order_slip->id);
            $fileData->setFileType($order_slip->file_type);

            if (get_class($apiResponse) == "stdClass") {
                try {
                    $fileData->setFileHash($apiResponse->file_hash);
                    $fileData->setBlockHash($apiResponse->block_hash);
                    $fileData->setStatus(1);
                    $fileData->add();
                } catch (PrestaShopException $e) {
                    var_dump($e->displayMessage());
                    die;
                }
            } else {
                $fileData->setStatus(0);
                $fileData->add();

                $apiError = new ApiError();
                $apiError->setIdOrderFile($order_slip->id);
                $apiError->setFileType($order_slip->file_type);
                $apiError->setCodeError($apiResponse->getCode());
                $apiError->setMessage($apiResponse->getMessage());
                $apiError->save();

                $notificationEmailsString = $kiwizApi->getApiCredential()->notification_emails;
                OrderSlip::sendEmail($notificationEmailsString, $apiResponse, $order_slip);
            }
        }

        return $res;
    }

    public static function sendEmail($notificationEmailsString, $apiResponse, $order_slip)
    {
        $notificationEmails = explode(';', $notificationEmailsString);

        $mail = new MailCore();
        $mail::Send(
            (int)Configuration::get('PS_LANG_DEFAULT'),
            'error_call_api',
            Mail::l('Error Call API'),
            array(
                '{idOrderFile}' => $order_slip->id,
                '{fileType}' => $order_slip->file_type,
                '{code}' => $apiResponse->getCode(),
                '{message}' => $apiResponse->getMessage()
            ),
            $notificationEmails,
            null,
            null,
            null,
            null,
            null,
            _PS_MODULE_DIR_ . 'kiwizinvoicehandler/mails/'
        );
    }

    public function update($null_values = false)
    {
        parent::update($null_values);

        $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi(Context::getContext()->smarty);
        $fileData = $this->getKiwizData();
        $apiResponse = $kiwizApi->save('creditmemo', $this);

        if (!$fileData) {
            $fileData = new FileData();
        }
        $fileData->setIdOrderFile($this->id);
        $fileData->setFileType($this->file_type);

        if (get_class($apiResponse) == "stdClass") {
            try {
                $fileData->setFileHash($apiResponse->file_hash);
                $fileData->setBlockHash($apiResponse->block_hash);
                $fileData->setStatus(1);
                $fileData->save();
            } catch (Exception $e) {
                $this->vd($e->displayMessage());
                die;
            }
        } else {
            $fileData->setStatus(0);
            $fileData->save();

            $apiError = new ApiError();
            $apiError->setIdOrderFile($this->id);
            $apiError->setFileType($this->file_type);
            $apiError->setCodeError($apiResponse->getCode());
            $apiError->setMessage($apiResponse->getMessage());
            $apiError->save();

            $notificationEmailsString = $kiwizApi->getApiCredential()->notification_emails;
            OrderSlip::sendEmail($notificationEmailsString, $apiResponse, $this);

            return $apiError;
        }

        return true;
    }

    public function getKiwizData()
    {
        $sql = "SELECT *
			FROM `" . _DB_PREFIX_ . "kiwiz_file_data`
			WHERE id_order_file = " . (int)$this->id .
            " AND file_type = " . "'slip'";
        $result = Db::getInstance()->getRow($sql);
        if ($result) {
            $fileData = new FileData();
            $fileData->setId($result['id']);
            $fileData->setIdOrderFile($result['id_order_file']);
            $fileData->setFileHash($result['file_hash']);
            $fileData->setBlockHash($result['block_hash']);
            $fileData->setFileType($result['file_type']);
            $fileData->setStatus($result['status']);

            return $fileData;
        }

        return $result;
    }

    public function vd($var)
    {
        echo '<pre style="color:#fff;background: #211f1f;padding: 10px;">';
        var_dump($var);
        echo '</pre>';
    }
}
