<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class AdminOrdersController extends AdminOrdersControllerCore
{
    public $fields_list;

    public $tpl_view_vars;

    public function __construct()
    {
        parent::__construct();
    }

    public function renderView()
    {
        parent::renderView();
        $order = $this->tpl_view_vars['order'];
        $kiwizService = new KiwizInvoiceHandler();
        $kiwizOrderStatus = $kiwizService->getOrderStatus($order->getDocuments());
        $invoiceNumber = (int)$order->invoice_number;
        if ($invoiceNumber > 0) {
            $orderInvoice = $order->getMainInvoice();
            $kiwizData = $orderInvoice->getKiwizData();
            $editable = false;
        } else {
            $kiwizData = false;
            $editable = true;
        }

        $this->context->smarty->assign('kiwizOrderStatus', $kiwizOrderStatus);
        $this->context->smarty->assign('mainKiwizData', $kiwizData);
        $this->context->smarty->assign('editable', $editable);

        return parent::renderView();
    }

    public function printPDFIcons($id_order, $tr)
    {
        static $valid_order_state = array();
        $order = new Order($id_order);
        if (!Validate::isLoadedObject($order)) {
            return '';
        }
        if (!isset($valid_order_state[$order->current_state])) {
            $valid_order_state[$order->current_state] = Validate::isLoadedObject($order->getCurrentOrderState());
        }
        if (!$valid_order_state[$order->current_state]) {
            return '';
        }
        $invoiceNumber = (int)$order->invoice_number;
        if ($invoiceNumber > 0) {
            $orderInvoice = $order->getMainInvoice();
            $kiwizData = $orderInvoice->getKiwizData();
        } else {
            $kiwizData = false;
        }

        $this->context->smarty->assign('kiwizData', $kiwizData);
        $this->context->smarty->assign(array(
            'order' => $order,
            'tr' => $tr
        ));
//        $this->setTemplate(_PS_MODULE_DIR_ . '/kiwizinvoicehandler/views/templates/admin/_print_pdf_icon.tpl');
        return $this->createTemplate('_print_pdf_icon.tpl')->fetch();
    }

    /**
     * @param Order $order
     * @return array
     */
    public function getProducts($order)
    {
        $products = $order->getProducts();

        foreach ($products as &$product) {
            if ($product['image'] != null) {
                $name =
                    'product_mini_'
                    . (int)$product['product_id']
                    . (isset($product['product_attribute_id']) ? '_'
                        . (int)$product['product_attribute_id'] : '')
                    . '.jpg';
                // generate image cache, only for back office
                $product['image_tag'] =
                    ImageManager::thumbnail(
                        _PS_IMG_DIR_ . 'p/' . $product['image']->getExistingImgPath() . '.jpg',
                        $name,
                        45,
                        'jpg'
                    );

                $product['my_image_tag']['url'] = '../img/tmp/';
                $product['my_image_tag']['name'] = $name;
                $product['my_image_tag']['time'] = '?time=' . time();

                if (file_exists(_PS_TMP_IMG_DIR_ . $name)) {
                    $product['image_size'] = getimagesize(_PS_TMP_IMG_DIR_ . $name);
                } else {
                    $product['image_size'] = false;
                }
            }
        }

        ksort($products);

        return $products;
    }
}
